% Newton's method
%
% ff is the function to optimize
%     called as "[ value, gradient, hessian ] = ff( xx )"
% initial_xx is the starting point
% line_search is the line search routine
%     called as "tt = line_search( ff, xx, direction )"
% epsilon is the stopping threshold
%
% xx is the final iterate
% values is the vector of function values (one for each iterate)
% runtimes is the vector of cumulative runtimes (one for each iterate)
% xxs is the matrix of iterates
%
function [ xx, values, runtimes, xxs ] = algorithm_newton( ff, initial_xx, line_search, epsilon )

	maximum_iterations = 65536;

	mm = length( initial_xx );
	if ( epsilon <= 0.0 ); error( 'epsilon must be positive' ); end

	values = [];
	runtimes = [];
	xxs = [];

	tic;

	xx = initial_xx;

	iterations = 0;
	while true

		[ value, gradient, hessian ] = ff( xx );

		% only compute the necessary return values
		if ( nargout > 1 )
			values = [ values, value ];
			if ( nargout > 2 )
				runtimes = [ runtimes, toc ];
				if ( nargout > 3 )
					xxs = [ xxs, xx ];
				end
			end
		end

		direction = -( hessian \ gradient );

		newton_decrement_squared = -gradient' * direction;
		if ( newton_decrement_squared <= epsilon ); break; end

		tt = line_search( ff, xx, direction );

		xx = xx + tt * direction;

		iterations = iterations + 1;
		if ( iterations > maximum_iterations ); error( 'Too many iterations' ); end

	end

end
