% Backtracking line search
%
% ff is the function to optimize
%     called as "[ value, gradient ] = ff( xx )"
% xx is the current iterate
% direction is the direction along which to perform the line search
% alpha is the alpha parameter to backtracking line search
% beta is the beta parameter to backtracking line search
%
% tt is the resulting step length
%
function tt = line_search_backtracking( ff, xx, direction, alpha, beta )

	maximum_iterations = 65536;

	% check parameter bounds
	if ( alpha <= 0.0   ); error( 'alpha must be positive'      ); end
	if ( 0.5   <= alpha ); error( 'alpha must be less that 0.5' ); end
	if ( beta <= 0.0  ); error( 'beta must be positive'      ); end
	if ( 1.0  <= beta ); error( 'beta must be less that 0.5' ); end

	[ value, gradient ] = ff( xx );
	derivative = direction' * gradient;

	if ( derivative >= 0.0 )

		% if we're not given a descent direction, we shouldn't go anywhere
		tt = 0.0;

	else

		iterations = 0;
		tt = 1.0;
		while true

			if ( ff( xx + tt * direction ) <= value + alpha * tt * derivative ); break; end

			tt = tt * beta;

			iterations = iterations + 1;
			if ( iterations >= maximum_iterations ); error( 'too many iterations' ); end

		end
	end

end
