% Log barrier objective
%
% ff is the objective function
% phi is the log-barrier constraint function
% xx is the current iterate
% tt is the scale of the log barrier
%
% value is the function value at xx
% gradient is the gradient at xx
% hessian is the hessian at xx
%
function [ value, gradient, hessian ] = objective_log_barrier( ff, phi, xx, tt )

	if ( nargout < 2 )
		[ ff_value ] = ff( xx );
		[ phi_value ] = phi( xx );
		value = tt * ff_value + phi_value;
	elseif ( nargout < 3 )
		[ ff_value, ff_gradient ] = ff( xx );
		[ phi_value, phi_gradient ] = phi( xx );
		value = tt * ff_value + phi_value;
		gradient = tt * ff_gradient + phi_gradient;
	else
		[ ff_value, ff_gradient, ff_hessian ] = ff( xx );
		[ phi_value, phi_gradient, phi_hessian ] = phi( xx );
		value = tt * ff_value + phi_value;
		gradient = tt * ff_gradient + phi_gradient;
		hessian = tt * ff_hessian + phi_hessian;
	end

end
