% Quadratic objective 0.5 * xx' * QQ * xx + vv' * xx with linear log-barrier constraints AA * xx + bb >= 0
%
% QQ is the Hessian matrix
% vv is the vector of linear coefficients
% AA is the constraint matrix
% bb is the constraint vector
% xx is the current iterate
% tt is the log-barrier scale
%
% value is the function value at xx
% gradient is the gradient at xx
% hessian is the hessian at xx
%
function [ value, gradient, hessian ] = objective_quadratic_log_barrier( QQ, vv, AA, bb, xx, tt )

	% we need to make sure that we get infinity (not a complex number) for xx which
	% violate the constraints
	function ll = safelog( pp )
		ll = log( max( 0, pp ) );
	end


	value = tt * ( 0.5 * xx' * QQ * xx + vv' * xx ) - sum( safelog( AA * xx + bb ) );

	if ( nargout > 1 )

		%gradient = ( TODO: gradient );

		if ( nargout > 2 )
			%hessian = ( TODO: hessian );
		end
	end

end
